﻿using System;
using System.ComponentModel;
using System.IO;
using System.Text.RegularExpressions;

using dotless.Core.Parser;
using dotless.Core.Parser.Infrastructure;

using EnvDTE;

using SamplePlugin.Lib;

using Shimmy.Api;
using Shimmy.Api.Utils;

namespace SamplePlugin
{
    public class DotLessProcessorAction : ActionBase
    {
        private Parser _Parser;

        /// <summary>
        /// Gets or sets the plugin context.
        /// </summary>
        /// <value>The plugin context.</value>
        protected PluginContext PluginContext { get; set; }

        /// <summary>
        /// Gets the name of the action.
        /// </summary>
        /// <value>The name.</value>
        public override string ActionName
        {
            get { return "DotLess Processor"; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not to minify the output of dotless.
        /// </summary>
        [Description("Set to True to cause files to minified when converted")]
        [DefaultValue(false)]
        public bool MinifyOutput { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether or not to resolve imports relative to the .less file.
        /// </summary>
        [Description("If true, the generated CSS file will be nested under the .less file - like a code-behind")]
        [DefaultValue(true)]
        public bool NestCssFile { get; set; }

        /// <summary>
        /// Gets or sets the regex pattern.
        /// </summary>
        /// <value>The regex pattern.</value>
        [Description("The regular expression pattern to match files that will trigger this action.  Action is triggered when the document(s) are saved.")]
        [DefaultValue(@"\.less")]
        public string RegexPattern { get; set; }

        /// <summary>
        /// Gets the parser.
        /// </summary>
        /// <value>The parser.</value>
        private Parser Parser { get { return _Parser ?? (_Parser = new Parser()); } }

        /// <summary>
        /// Initializes a new instance of the <see cref="DotLessProcessorAction"/> class.
        /// </summary>
        public DotLessProcessorAction()
        {
            RegexPattern = @"\.less";
            Enabled = false;
            MinifyOutput = false;
            NestCssFile = true;
        }

        /// <summary>
        /// Initializes the action.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public override void Initialize(PluginContext pluginContext)
        {
            PluginContext = pluginContext;
            pluginContext.ApplicationEvents.DocumentEvents.DocumentSaved += DocumentSaved;
        }

        /// <summary>
        /// Unloads this action.  Clean up any resources/events.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public override void Unload(PluginContext pluginContext)
        {
            pluginContext.ApplicationEvents.DocumentEvents.DocumentSaved -= DocumentSaved;
        }

        private void DocumentSaved(Document document)
        {
            if (Regex.IsMatch(document.FullName, RegexPattern, RegexOptions.IgnoreCase))
            {
                var textDocument = document.Object() as TextDocument;
                if (textDocument != null)
                {
                    try
                    {
                        string transformedText = Lessify(textDocument);
                        SaveFile(document, transformedText);
                    }
                    catch (Exception exception)
                    {
                        ErrorHandler.HandleError(exception.Message);
                    }
                }
            }
        }

        private void SaveFile(Document document, string transformedText)
        {
            // Get file name
            var fileName = Path.GetFileNameWithoutExtension(document.FullName) + ".css";

            // Add file to project
            var projectHelper = new ProjectHelper(PluginContext);
            ProjectItems parent = NestCssFile ? document.ProjectItem.ProjectItems : document.ProjectItem.ContainingProject.ProjectItems;
            projectHelper.AddFile(Path.Combine(document.Path, fileName), transformedText, parent);
        }

        private string Lessify(TextDocument textDocument)
        {
            string documentText = textDocument.CreateEditPoint().GetText(textDocument.EndPoint);
            var documentDirectory = new DirectoryInfo(Path.GetDirectoryName(textDocument.Parent.FullName));
            
            using (new DirectoryContext(documentDirectory))
            {
                var css = string.Empty;
                var ruleset = Parser.Parse(documentText, textDocument.Parent.FullName);
                css = ruleset.ToCSS(new Env { Compress = MinifyOutput });
                return css;
            }
        }

    }
}
