﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;

namespace Shimmy.Api
{
    /// <summary>
    /// The base action from which all action should derive
    /// </summary>
    [InheritedExport(typeof(IAction))]
    public abstract class ActionBase : IAction
    {
        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="IAction"/> is enabled.
        /// </summary>
        /// <value><c>true</c> if enabled; otherwise, <c>false</c>.</value>
        [Category("Basic Settings")]
        [DefaultValue(true)]
        [Description("Toggles this action on and off.  The action will need to be reloaded for changes to take effect")]
        public bool Enabled { get; set; }

        /// <summary>
        /// Gets the name of the action.B
        /// </summary>
        /// <value>The name.</value>
        [Category("Basic Settings")]
        [Description("The friendly name of the action. For reference only.")]
        public abstract string ActionName { get; }

        /// <summary>
        /// Initializes a new instance of the <see cref="ActionBase"/> class.
        /// </summary>
        protected ActionBase()
        {
            Enabled = true;
        }

        /// <summary>
        /// Initializes the action.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public abstract void Initialize(PluginContext pluginContext);

        /// <summary>
        /// Unloads this action.  Clean up any resources/events.
        /// </summary>
        /// <remarks>
        /// <para>
        ///  It is possible to receive an <see cref="Unload"/> request without an <see cref="Initialize"/> in the case
        ///  of disabled actions.  Since actions can be disabled after they've been
        ///  initialized, we must still run unload regardless of the <see cref="Enabled"/> state.
        ///  If the action starts disabled, then we will have never run the initialization.
        ///  </para>
        ///  <para>
        ///  As such, take care to perform proper object state checks (null checks) or set a flag
        ///  when the <see cref="Initialize"/> runs.
        /// </para>
        /// </remarks>
        /// <param name="pluginContext">The plugin context.</param>
        public abstract void Unload(PluginContext pluginContext);
    }
}
