﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

using EnvDTE;

using EnvDTE80;

using Microsoft.VisualStudio.CommandBars;

namespace Shimmy.Api.Utils
{
    /// <summary>
    /// Menu builder
    /// </summary>
    public class CommandMenu : IDisposable
    {
        private readonly CommandBar _Root;
        private readonly DTE2 _Application;
        private readonly AddIn _AddIn;
        private readonly Stack<CommandBar> _MenuStack;
        private readonly List<Command> _Commands;
        private readonly List<CommandBarPopup> _SubMenus;

        /// <summary>
        /// Gets the current menu.
        /// </summary>
        /// <value>The current menu.</value>
        private CommandBar CurrentMenu { get { return _MenuStack.Peek(); } }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommandMenu"/> class.
        /// </summary>
        /// <param name="root">The root.</param>
        /// <param name="context">The context.</param>
        public CommandMenu(CommandBar root, PluginContext context)
        {
            _Root = root;
            _Application = context.Application;
            _AddIn = context.AddIn;
            _MenuStack = new Stack<CommandBar>();
            _MenuStack.Push(_Root);
            _Commands = new List<Command>();
            _SubMenus = new List<CommandBarPopup>();
        }

        /// <summary>
        /// Creates a sub menu
        /// </summary>
        /// <param name="caption">The caption.</param>
        /// <param name="append">if set to <c>true</c> the sub menu will be appended to the parent.</param>
        /// <param name="beginGroup">if set to <c>true</c> a group will be started (creating a separator)</param>
        /// <returns></returns>
        public CommandMenu SubMenu(string caption, bool append = false, bool beginGroup = false)
        {
            
            CommandBarPopup subMenu = append ? CommandBarUtils.AppendSubMenu(CurrentMenu, caption) : CommandBarUtils.PrependSubMenu(CurrentMenu, caption);
            subMenu.BeginGroup = beginGroup;
            _SubMenus.Add(subMenu);
            _MenuStack.Push(subMenu.CommandBar);
            return this;
        }

        /// <summary>
        /// Creates and adds the given command to the menu
        /// </summary>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="caption">The caption.</param>
        /// <param name="append">if set to <c>true</c> the command will be appended to the parent.</param>
        /// <param name="description">The description.</param>
        /// <param name="icon">The icon.</param>
        /// <param name="beginGroup">if set to <c>true</c> [begin group].</param>
        /// <returns></returns>
        public CommandMenu Command(string commandName, string caption, bool append = false, string description = null, Icon icon = null, bool beginGroup = false)
        {
            var command = _Application.Commands.AddNamedCommand(_AddIn, commandName, caption, description ?? caption, true, 0, null, 3);
            _Commands.Add(command);

            // Create the button
            var position = 1;
            if (append)
                position = CurrentMenu.Controls.Count + 1;
            var button = (CommandBarButton)command.AddControl(CurrentMenu, position);
            if (icon != null)
            {
                button.Picture = ImageConverter.ToStdPicture(icon.ToBitmap());
            }
            button.BeginGroup = beginGroup;
            button.Caption = caption;
            return this;
        }

        /// <summary>
        /// Ends the menu.
        /// </summary>
        /// <returns></returns>
        public CommandMenu EndMenu()
        {
            if (_MenuStack.Count > 1)
            {
                _MenuStack.Pop();
            }
            return this;
        }

        /// <summary>
        /// Deletes this instance.
        /// </summary>
        public void Delete()
        {
            try
            {
                foreach (var command in _Commands)
                {
                    command.Delete();
                }
                _Commands.Clear();
            }
            catch { }

            try
            {
                foreach (var subMenu in _SubMenus)
                {
                    subMenu.Delete();
                }
                _SubMenus.Clear();
            }
            catch { }
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Delete();
            GC.SuppressFinalize(this);
        }
    }
}
