﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

using EnvDTE;

namespace Shimmy.Api.Utils
{
    /// <summary>
    /// Project helper class
    /// </summary>
    public class ProjectHelper
    {
        private readonly PluginContext _PluginContext;

        /// <summary>
        /// Initializes a new instance of the <see cref="ProjectHelper"/> class.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public ProjectHelper(PluginContext pluginContext)
        {
            _PluginContext = pluginContext;
        }

        /// <summary>
        /// Gets the selected project items.
        /// </summary>
        public IList<ProjectItem> GetSelectedProjectItems()
        {
            List<ProjectItem> projectItems = null;
            var solutionExplorer = (UIHierarchy)_PluginContext.Application.Windows.Item(Dte.Constants.vsWindowKindSolutionExplorer).Object;
            if (solutionExplorer != null)
            {
                if (solutionExplorer.SelectedItems != null && ((UIHierarchyItem[])solutionExplorer.SelectedItems).Length > 0)
                {
                    projectItems = new List<ProjectItem>();
                    foreach (UIHierarchyItem selectedItem in (UIHierarchyItem[])solutionExplorer.SelectedItems)
                    {
                        projectItems.Add((ProjectItem)selectedItem.Object);
                    }
                }
            }
            return projectItems;
        }

        /// <summary>
        /// Adds the given file to the project.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <param name="content">The content.</param>
        /// <param name="parent">The parent.</param>
        public void AddFile(string fileName, string content, ProjectItems parent)
        {
            if (File.Exists(fileName))
            {
                // Update file
                UpdateFile(fileName, content);
            }
            else
            {
                File.WriteAllText(fileName, content);
            }
            parent.AddFromFile(fileName);
        }

        /// <summary>
        /// Determines whether the specified file is under source control.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public bool IsFileUnderSourceControl(string fileName)
        {
            var sourceControl = _PluginContext.Application.SourceControl;
            return sourceControl != null && sourceControl.IsItemUnderSCC(fileName);
        }

        private void UpdateFile(string fileName, string content)
        {
            var originalContent = File.ReadAllText(fileName);
            if (originalContent != content)
            {
                // New content
                CheckOut(fileName);
                File.WriteAllText(fileName, content);
            }
        }

        private void CheckOut(string fileName)
        {
            var sourceControl = _PluginContext.Application.SourceControl;
            if (IsFileUnderSourceControl(fileName) && !sourceControl.IsItemCheckedOut(fileName))
            {
                var task = new TaskFactory().StartNew(() => { sourceControl.CheckOutItem(fileName); });
                task.Wait();
            }
        }
    }
}
