﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;

using Shimmy.Lib;
using Shimmy.Properties;

namespace Shimmy
{
    [RunInstaller(true)]
    public class AddInInstaller : Installer
    {
        private const string MANIFEST_PATH_KEY = "ManifestPath";
        private const string ADDIN_INSTALL_SUBDIR = @"Microsoft\MSEnvShared\Addins\";

        private string AssemblyPath
        {
            get { return Context.Parameters["AssemblyPath"]; }
        }

        private string AddInManifestName
        {
            get { return Path.GetFileNameWithoutExtension(Context.Parameters["AssemblyPath"]) + ".AddIn"; }
        }

        /// <summary>
        /// When overridden in a derived class, performs the installation.
        /// </summary>
        /// <param name="stateSaver">An <see cref="T:System.Collections.IDictionary"/> used to save information needed to perform a commit, rollback, or uninstall operation.</param>
        /// <exception cref="T:System.ArgumentException">The <paramref name="stateSaver"/> parameter is null. </exception>
        /// <exception cref="T:System.Exception">An exception occurred in the <see cref="E:System.Configuration.Install.Installer.BeforeInstall"/> event handler of one of the installers in the collection.-or- An exception occurred in the <see cref="E:System.Configuration.Install.Installer.AfterInstall"/> event handler of one of the installers in the collection. </exception>
        public override void Install(IDictionary stateSaver)
        {
            base.Install(stateSaver);
            try
            {
                // Get our target and make sure it exists
                var addInTargetPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), ADDIN_INSTALL_SUBDIR);
                var directory = new DirectoryInfo(addInTargetPath);
                if (!directory.Exists)
                {
                    directory.Create();
                }

                // Write out the manifest
                string manifestFilePath = Path.Combine(addInTargetPath, AddInManifestName);
                var manifest = GetAddinManifest();
                File.WriteAllText(manifestFilePath, manifest);

                // Store the manifest path for undo
                stateSaver.Add(MANIFEST_PATH_KEY, manifestFilePath);
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }
        }

        /// <summary>
        /// When overridden in a derived class, restores the pre-installation state of the computer.
        /// </summary>
        /// <param name="savedState">An <see cref="T:System.Collections.IDictionary"/> that contains the pre-installation state of the computer.</param>
        /// <exception cref="T:System.ArgumentException">The <paramref name="savedState"/> parameter is null.-or- The saved-state <see cref="T:System.Collections.IDictionary"/> might have been corrupted. </exception>
        /// <exception cref="T:System.Configuration.Install.InstallException">An exception occurred during the <see cref="M:System.Configuration.Install.Installer.Rollback(System.Collections.IDictionary)"/> phase of the installation. This exception is ignored and the rollback continues. However, the computer might not be fully reverted to its initial state after the rollback completes. </exception>
        public override void Rollback(IDictionary savedState)
        {
            base.Rollback(savedState);
            try
            {
                // Delete the manifest
                var fileName = (string)savedState[MANIFEST_PATH_KEY];
                if (File.Exists(fileName))
                {
                    File.Delete(fileName);
                }
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }
        }

        /// <summary>
        /// When overridden in a derived class, removes an installation.
        /// </summary>
        /// <param name="savedState">An <see cref="T:System.Collections.IDictionary"/> that contains the state of the computer after the installation was complete.</param>
        /// <exception cref="T:System.ArgumentException">The saved-state <see cref="T:System.Collections.IDictionary"/> might have been corrupted. </exception>
        /// <exception cref="T:System.Configuration.Install.InstallException">An exception occurred while uninstalling. This exception is ignored and the uninstall continues. However, the application might not be fully uninstalled after the uninstallation completes. </exception>
        public override void Uninstall(IDictionary savedState)
        {
            base.Uninstall(savedState);

            try
            {
                var fileName = (string)savedState[MANIFEST_PATH_KEY];
                if (File.Exists(fileName))
                {
                    File.Delete(fileName);
                }
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }
        }

        private string GetAddinManifest()
        {
            string addInAssemblyPath = AssemblyPath;
            string addInManifestResourceName = string.Format("{0}.Manifest.{1}", Assembly.GetExecutingAssembly().GetName().Name, AddInManifestName);

            string manifest;
            using (Stream resourceStream = Assembly.GetExecutingAssembly().GetManifestResourceStream(addInManifestResourceName))
            {
                var xmlDocument = XDocument.Load(XmlReader.Create(resourceStream));
                foreach (var element in xmlDocument.Descendants().Elements())
                {
                    switch (element.Name.LocalName)
                    {
                        case "Assembly":
                            element.SetValue(addInAssemblyPath);
                            continue;
                        case "FriendlyName":
                            element.SetValue(string.Format(Resources.Addin_FriendlyName, PluginEnvironment.Constants.PRODUCT_VERSION));
                            continue;
                        case "Description":
                            element.SetValue(string.Format(Resources.Addin_Description, PluginEnvironment.Constants.PRODUCT_VERSION));
                            continue;
                        case "AboutBoxDetails":
                            element.SetValue(string.Format(Resources.Addin_AboutBoxDetails, DateTime.Now.Year));
                            continue;
                        case "Category":
                            element.SetAttributeValue("Name", Resources.Addin_OptionsCategoryName);
                            continue;
                        case "SubCategory":
                            element.SetAttributeValue("Name", Resources.Addin_OptionsSubCategoryName);
                            continue;
#if DEBUG
                        case "LoadBehavior":
                            element.SetValue(0);
                            continue;
#endif
                    }
                }
                manifest = xmlDocument.ToString();
            }
            return manifest;
        }

        [Conditional("DEBUG")]
        private static void HandleError(Exception exception)
        {
            MessageBox.Show(exception.ToString());
            Debugger.Launch();
        }
    }
}
