﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using EnvDTE;

using Shimmy.Api;
using Shimmy.Lib;
using Shimmy.Lib.Utils;
using Shimmy.Properties;

namespace Shimmy
{
    public partial class Configuration : UserControl, IDTToolsOptionsPage
    {
        private List<IAction> _ChangedActions;
        private readonly GlobalSettings _GlobalSettings;

        public Configuration()
        {
            InitializeComponent();
            _GlobalSettings = GlobalSettings.Load();
        }

        public void OnAfterCreated(DTE dteObject)
        {
            ActionConfigurationGroupBox.Enabled = true;
            PluginSourceDirectoryTextBox.Text = _GlobalSettings.ConfigurationDirectoryLocations ?? string.Empty;
            if (PluginEnvironment.Current == null)
            {
                ActionConfigurationGroupBox.Enabled = false;
            }
            else
            {
                ActionComboBox.Items.Clear();
                if (PluginEnvironment.Current.ActionList.Count > 0)
                {
                    foreach ( var action in PluginEnvironment.Current.ActionList )
                    {
                        var displayName = string.Format( "{0} ({1})", action.ActionName, action.GetType().FullName );
                        ActionComboBox.Items.Add( new ComboBoxItem( displayName, action ) );
                    }
                    ActionComboBox.DisplayMember = "DisplayName";
                    ActionComboBox.SelectedIndex = 0;
                }

                ReloadActionsCheckBox.Checked = false;
                ReloadActionsCheckBox.Enabled = false;
            }
            _ChangedActions = new List<IAction>();
        }

        public void GetProperties(ref object propertiesObject)
        {
            propertiesObject = null;
        }

        public void OnOK()
        {
            SavePluginDirectories();

            _GlobalSettings.Save();
            foreach (IAction action in _ChangedActions)
            {
                SettingsManager.Instance.Persist(action);
                if (ReloadActionsCheckBox.Enabled && ReloadActionsCheckBox.Checked)
                {
                    PluginEnvironment.Current.ReloadAction(action);
                }
            }
        }

        public void OnCancel()
        {
            if (_ChangedActions.Count > 0)
            {
                if (MessageBox.Show(Resources.ConfirmAbandonChanges, Resources.AddInErrorMessageBoxTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    OnOK();
                }
            }
        }

        public void OnHelp()
        {
            MessageBox.Show(Resources.ConfigurationHelpText, Resources.ConfigurationHelpCaption, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void SavePluginDirectories()
        {
            if (ValidatePluginDirectories())
            {
                _GlobalSettings.ConfigurationDirectoryLocations = PluginSourceDirectoryTextBox.Text;
            }
        }

        private bool ValidatePluginDirectories()
        {
            bool canSaveDirectories = true;
            if (!string.IsNullOrWhiteSpace(PluginSourceDirectoryTextBox.Text))
            {
                string invalidDirectories = string.Empty;
                foreach (var dir in PluginSourceDirectoryTextBox.Text.Split('\r', '\n'))
                {
                    if (!FileSystemUtils.Exists(new DirectoryInfo(dir)))
                    {
                        invalidDirectories += dir + Environment.NewLine;
                    }
                }
                if (invalidDirectories.Length > 0)
                {
                    string warning = string.Format(Resources.ConfigurationInvalidDirectories, Environment.NewLine + invalidDirectories);
                    if (MessageBox.Show(warning, Resources.AddInErrorMessageBoxTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) != DialogResult.Yes)
                    {
                        canSaveDirectories = false;
                    }
                }
            }
            return canSaveDirectories;
        }

        private void SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ActionComboBox.SelectedItem != null)
            {
                ActionPropertyGrid.SelectedObject = ((ComboBoxItem)ActionComboBox.SelectedItem).Action;
            }
            else
            {
                ActionPropertyGrid.SelectedObject = null;
            }
        }

        private void ActionPropertyGridPropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            _ChangedActions.Add(ActionPropertyGrid.SelectedObject as IAction);
            ReloadActionsCheckBox.Enabled = true;
            ReloadActionsCheckBox.Checked = true;
        }

        private class ComboBoxItem
        {
            public string DisplayName { get; set; }
            public IAction Action { get; set; }

            public ComboBoxItem(string key, IAction action)
            {
                DisplayName = key;
                Action = action;
            }
        }
    }
}
