﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Linq;

using Shimmy.Api;
using Shimmy.Properties;

namespace Shimmy.Lib
{
    internal class PluginEnvironment
    {
        public class Constants
        {
            public const string PRODUCT_NAME = "Shimmy";
            public const string PRODUCT_VERSION = "1.0.4";
            public const string SETTINGS_REG_KEY = @"SOFTWARE\Microsoft\VisualStudio\10.0\" + PRODUCT_NAME;
        }

#pragma warning disable 649
        [ImportMany(typeof(IAction))]
        private IEnumerable<Lazy<IAction>> _Actions; // Imported via MEF

        [ImportMany(typeof(ICommandHandler))]
        private IEnumerable<Lazy<ICommandHandler>> _CommandHandlers; // Imported via MEF
#pragma warning restore 649

        private CompositionContainer _Container;
        private readonly IList<IAction> _ActionList = new List<IAction>();
        private readonly IDictionary<string, IList<ICommandHandler>> _CommandHandlerRegistry = new Dictionary<string, IList<ICommandHandler>>();

        /// <summary>
        /// Gets or sets the current instance.
        /// </summary>
        /// <value>The instance.</value>
        public static PluginEnvironment Current { get; private set; }

        /// <summary>
        /// Occurs when the plugin import sync complete.
        /// </summary>
        public event EventHandler<PluginImportEventArgs> PluginImportSyncComplete;

        /// <summary>
        /// Gets or sets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public GlobalSettings Settings { get; set; }

        /// <summary>
        /// The base directory for plugins
        /// </summary>
        public static string BasePluginDirectory { get { return Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), Constants.PRODUCT_NAME + "\\Plugins"); } }

        /// <summary>
        /// Gets a value indicating whether this addin is loaded.
        /// </summary>
        /// <value><c>true</c> if this instance is loaded; otherwise, <c>false</c>.</value>
        public bool IsLoaded { get { return Context != null; } }

        /// <summary>
        /// Gets the command handler registry.
        /// </summary>
        /// <value>The command handler registry.</value>
        public IDictionary<string, IList<ICommandHandler>> CommandHandlerRegistry
        {
            get { return _CommandHandlerRegistry; }
        }

        /// <summary>
        /// Gets or sets the context.
        /// </summary>
        /// <value>The context.</value>
        public PluginContext Context { get; private set; }

        /// <summary>
        /// Gets the action list.
        /// </summary>
        /// <value>The action list.</value>
        public IList<IAction> ActionList
        {
            get { return _ActionList; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PluginEnvironment"/> class.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <param name="settings">The settings.</param>
        public PluginEnvironment(PluginContext context, GlobalSettings settings)
        {
            Context = context;
            Settings = settings;
        }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        public void Initialize()
        {
            // Register this as the current instance
            Current = this;

            //An aggregate catalog that combines multiple catalogs
            var catalog = new AggregateCatalog();

            // Adds all extensions found in the relative directory .\Plugins
            var pluginPath = BasePluginDirectory;
            if (Directory.Exists(pluginPath))
            {
                catalog.Catalogs.Add(new DirectoryCatalog(pluginPath));
            }

            // Get other plugin paths
            if (Settings.ConfigurationDirectories != null)
            {
                foreach (var configurationDirectory in Settings.ConfigurationDirectories)
                {
                    var manager = new PluginImportManager(configurationDirectory);

                    // Create a local copy, in case the source is remote
                    manager.SyncDirectory();

                    // Fire off a "begin" event
                    OnPluginImportSyncComplete(new PluginImportEventArgs(manager));

                    if (manager.PluginPath.Exists)
                    {
                        catalog.Catalogs.Add(new DirectoryCatalog(manager.PluginPath.FullName, "*.dll"));
                    }
                }
            }

            //Create the CompositionContainer with the parts in the catalog
            _Container = new CompositionContainer(catalog);

            //Fill the imports of this object
            try
            {
                _Container.ComposeParts(this);
            }
            catch (CompositionException compositionException)
            {
                DefaultErrorHandler.HandleError(compositionException, compositionException.ToString());
            }

            RegisterActions();
            RegisterCommandHandlers();
            InitializeActions();
        }

        /// <summary>
        /// Unloads all plugins
        /// </summary>
        public void Unload()
        {
            UnloadActions();
            ActionList.Clear();
            CommandHandlerRegistry.Clear();
            Context = null;
            Settings = null;
            _Container.Dispose();
        }

        /// <summary>
        /// Reloads the action.
        /// </summary>
        /// <param name="action">The action.</param>
        public void ReloadAction(IAction action)
        {
            action.Unload(Context);
            if (action.Enabled)
            {
                action.Initialize(Context);
            }
        }

        /// <summary>
        /// Raises the <see cref="PluginImportSyncComplete"/> event.
        /// </summary>
        /// <param name="e">The <see cref="Shimmy.Lib.PluginImportEventArgs"/> instance containing the event data.</param>
        protected void OnPluginImportSyncComplete(PluginImportEventArgs e)
        {
            EventHandler<PluginImportEventArgs> handler = PluginImportSyncComplete;
            if (handler != null)
                handler(this, e);
        }

        private void InitializeActions()
        {
            if (ActionList.Count > 0)
            {
                foreach (var action in ActionList)
                {
                    try
                    {
                        SettingsManager.Instance.Restore(action);
                        if (action.Enabled)
                        {
                            action.Initialize(Context);
                        }
                    }
                    catch (Exception exception)
                    {
                        DefaultErrorHandler.HandleError(exception, string.Format(Resources.ErrorInitializingAction, action.ActionName, exception));
                    }
                }
            }
        }

        private void UnloadActions()
        {
            // Unload all actions
            if (ActionList != null && ActionList.Count > 0)
            {
                foreach (var action in ActionList)
                {
                    try
                    {
                        action.Unload(Context);
                    }
                    catch (Exception exception)
                    {
                        DefaultErrorHandler.HandleError(exception, string.Format(Resources.ErrorUnloadingAction, action.ActionName, exception));
                    }
                }
            }
        }

        private void RegisterActions()
        {
            // Initialize all actions
            if (_Actions != null && _Actions.Any())
            {
                _ActionList.Clear();
                foreach (var action in _Actions)
                {
                    _ActionList.Add(action.Value);
                }
            }
        }

        private void RegisterCommandHandlers()
        {
            // Initialize all actions
            if (_CommandHandlers != null && _CommandHandlers.Any())
            {
                foreach (var handler in _CommandHandlers)
                {
                    var commandNames = handler.Value.CommandNames;
                    foreach (var name in commandNames)
                    {
                        var commandName = Context.AddIn.ProgID + "." + name;
                        if (!CommandHandlerRegistry.ContainsKey(commandName))
                        {
                            CommandHandlerRegistry.Add(commandName, new List<ICommandHandler>());
                        }
                        CommandHandlerRegistry[commandName].Add(handler.Value);
                    }
                }
            }
        }
    }

    internal class PluginImportEventArgs : EventArgs
    {
        public PluginImportManager Manager { get; private set; }

        public PluginImportEventArgs(PluginImportManager manager)
        {
            Manager = manager;
        }
    }
}
