﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.IO;

using Shimmy.Lib.Utils;

namespace Shimmy.Lib
{    
    /// <summary>
    /// Handles bringing in new and updated files from external sources
    /// </summary>
    internal class PluginImportCacheProvider
    {
        private readonly string _SourceDirectory;
        private readonly string _BasePluginPath;
        private Dictionary<string, string> _NewPlugins;
        private Dictionary<string, string> _UpdatedPlugins;

        /// <summary>
        /// Gets the new plugins.
        /// </summary>
        /// <value>The new plugins.</value>
        public IDictionary<string, string> NewPlugins
        {
            get { GetPendingPlugins(); return _NewPlugins; }
        }

        /// <summary>
        /// Gets the updated plugins.
        /// </summary>
        /// <value>The updated plugins.</value>
        public IDictionary<string, string> UpdatedPlugins
        {
            get { GetPendingPlugins(); return _UpdatedPlugins; }
        }

        public string CachePath { get { return PluginImportManager.GetPluginDirectoryPath(_BasePluginPath, _SourceDirectory); } }

        /// <summary>
        /// Initializes a new instance of the <see cref="PluginImportCacheProvider"/> class.
        /// </summary>
        /// <param name="sourceDirectory">The source directory.</param>
        /// <param name="basePluginPath">The base plugin path.</param>
        public PluginImportCacheProvider(string sourceDirectory, string basePluginPath)
        {
            _SourceDirectory = sourceDirectory;
            _BasePluginPath = basePluginPath;
        }

        /// <summary>
        /// Caches the source directory.
        /// </summary>
        public void CacheSourceDirectory()
        {
            ApplyUpdates();
            var sync = new FileSync(_SourceDirectory, CachePath);
            sync.BeforeCopy += BeforeCopy;
            sync.SyncSourceToDestination("*.dll");
        }

        private void ApplyUpdates()
        {
            if (Directory.Exists(CachePath))
            {
                var files = new DirectoryInfo( CachePath ).GetFiles();
                foreach ( var fileInfo in files )
                {
                    if ( fileInfo.Exists )
                    {
                        if ( fileInfo.FullName.EndsWith( ".update" ) )
                        {
                            try
                            {
                                var existingFileName = fileInfo.FullName.Replace( ".update", string.Empty );
                                File.Delete( existingFileName );
                                fileInfo.MoveTo( existingFileName );
                            }
                            catch ( IOException )
                            {
                                // Cannot update at this time.
                            }
                            catch ( UnauthorizedAccessException )
                            {
                                // Cannot update at this time.
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Moves files into staging directories.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="BeforeCopyEventArgs"/> instance containing the event data.</param>
        private void BeforeCopy(object sender, BeforeCopyEventArgs e)
        {
            var basePath = Path.GetDirectoryName(e.DestinationFile.FullName);
            var path = basePath;
            if (e.FileCopyReason != FileCopyReason.None)
            {

                path = Path.Combine(path, e.FileCopyReason.ToString());
            }
            Directory.CreateDirectory(path);
            e.DestinationFile = new FileInfo(Path.Combine(path, e.DestinationFile.Name));
        }

        private void GetPendingPlugins()
        {
            var newDirectory = new DirectoryInfo(Path.Combine(CachePath, FileCopyReason.New.ToString()));
            var updatedDirectory = new DirectoryInfo(Path.Combine(CachePath, FileCopyReason.Updated.ToString()));
            if (newDirectory.Exists)
            {
                var files = newDirectory.GetFiles();
                if (files.Length > 0)
                {
                    _NewPlugins = files.ToDictionary(x => x.FullName, x => Path.Combine(_SourceDirectory, Path.GetFileName(x.FullName)));
                }
            }
            if (updatedDirectory.Exists)
            {
                var files = updatedDirectory.GetFiles();
                if (files.Length > 0)
                {
                    _UpdatedPlugins = files.ToDictionary(x => x.FullName, x => Path.Combine(_SourceDirectory, Path.GetFileName(x.FullName)));
                }
            }
        }

        public void AcceptPlugin(string pluginPath)
        {
            var type = Path.GetFileName(Path.GetDirectoryName(pluginPath));
            if (type == FileCopyReason.New.ToString())
            {
                // It's a new plugin
                File.Move(pluginPath, Path.Combine(CachePath, Path.GetFileName(pluginPath)));
            }
            if (type == FileCopyReason.Updated.ToString())
            {
                // It's an updated plugin
                File.Move(pluginPath, Path.Combine(CachePath, Path.GetFileName(pluginPath) + ".update"));
            }
            ApplyUpdates();
        }
    }
}

