﻿using System.Collections.Generic;
using System.IO;

using Shimmy.Lib.Utils;

namespace Shimmy.Lib
{
    /// <summary>
    /// Handles bringing in new and updated files from external sources.  Mostly a 
    /// convenience wrapper around the cache provider.
    /// </summary>
    internal class PluginImportManager
    {
        private readonly DirectoryInfo _PluginDirectory;
        private readonly PluginImportCacheProvider _PluginCacheProvider;

        /// <summary>
        /// Gets the plugin path.
        /// </summary>
        /// <value>The plugin path.</value>
        public DirectoryInfo PluginPath { get { return new DirectoryInfo(_PluginCacheProvider.CachePath); } }

        /// <summary>
        /// Initializes a new instance of the <see cref="PluginImportManager"/> class.
        /// </summary>
        /// <param name="pluginDirectory">The plugin directory.</param>
        public PluginImportManager(string pluginDirectory)
            : this(new DirectoryInfo(pluginDirectory))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PluginImportManager"/> class.
        /// </summary>
        /// <param name="pluginDirectory">The plugin directory.</param>
        public PluginImportManager(DirectoryInfo pluginDirectory)
        {
            _PluginDirectory = pluginDirectory;
            _PluginCacheProvider = new PluginImportCacheProvider(_PluginDirectory.FullName, PluginEnvironment.BasePluginDirectory);
        }

        /// <summary>
        /// Gets the pending updates.
        /// </summary>
        /// <returns></returns>
        public IDictionary<string, string> GetPendingUpdates()
        {
            return _PluginCacheProvider.UpdatedPlugins;
        }

        /// <summary>
        /// Gets the pending plugins.
        /// </summary>
        /// <returns></returns>
        public IDictionary<string, string> GetPendingPlugins()
        {
            return _PluginCacheProvider.NewPlugins;
        }

        /// <summary>
        /// Allows the plugin.
        /// </summary>
        /// <param name="pluginPath">The plugin path.</param>
        public void AllowPlugin(string pluginPath)
        {
            _PluginCacheProvider.AcceptPlugin(pluginPath);
        }

        /// <summary>
        /// Syncs the directory.
        /// </summary>
        public void SyncDirectory()
        {
            _PluginCacheProvider.CacheSourceDirectory();
        }

        /// <summary>
        /// Gets the plugin directory path.
        /// </summary>
        /// <param name="basePluginPath">The base plugin path.</param>
        /// <param name="pluginDirectory">The plugin directory.</param>
        /// <returns></returns>
        public static string GetPluginDirectoryPath(string basePluginPath, string pluginDirectory)
        {
            return Path.Combine(basePluginPath, GetDirectoryId(pluginDirectory));
        }

        /// <summary>
        /// Gets the directory id.
        /// </summary>
        /// <param name="directory">The directory.</param>
        /// <returns></returns>
        private static string GetDirectoryId(string directory)
        {
            return StringUtils.GenerateHash(directory);
        }
    }
}
