﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Threading.Tasks;

namespace Shimmy.Lib.Utils
{
    internal class FileSync
    {
        private readonly DirectoryInfo _SourceDirectory;
        private readonly DirectoryInfo _Destination;
        public event EventHandler<BeforeCopyEventArgs> BeforeCopy;

        /// <summary>
        /// Initializes a new instance of the <see cref="FileSync"/> class.
        /// </summary>
        /// <param name="sourceDirectory">The source directory.</param>
        /// <param name="destination">The destination.</param>
        public FileSync(string sourceDirectory, string destination)
            : this(new DirectoryInfo(sourceDirectory), new DirectoryInfo(destination))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileSync"/> class.
        /// </summary>
        /// <param name="sourceDirectory">The source directory.</param>
        /// <param name="destination">The destination.</param>
        public FileSync(DirectoryInfo sourceDirectory, DirectoryInfo destination)
        {
            _SourceDirectory = sourceDirectory;
            _Destination = destination;
        }

        /// <summary>
        /// Syncs the source and destination to make the destination look exactly like the source. <b>Files may be deleted</b>
        /// </summary>
        /// <param name="pattern">The pattern.</param>
        /// <param name="throwOnError">if set to <c>true</c> exceptions will be thrown and not swallowed.</param>
        /// <param name="async">if set to <c>true</c> the method will perform the copy asynchronously.</param>
        public void SyncSourceToDestination(string pattern, bool throwOnError = false, bool async = false, bool deleteDestinationFilesNotInSource = true)
        {
            if(!FileSystemUtils.Exists(_SourceDirectory))
            {
                return;
            }
            CopyFromSource(pattern, throwOnError, async);
            if (deleteDestinationFilesNotInSource)
                DeleteDestinationFilesNotInSource(pattern, throwOnError, async);
        }

        /// <summary>
        /// Copies files from source to the destination.  No files are deleted.
        /// </summary>
        /// <param name="pattern">The pattern.</param>
        /// <param name="throwOnError">if set to <c>true</c> exceptions will be thrown and not swallowed.</param>
        /// <param name="async">if set to <c>true</c> the method will perform the copy asynchronously.</param>
        protected void CopyFromSource(string pattern, bool throwOnError = false, bool async = false)
        {
            var files = _SourceDirectory.EnumerateFiles(pattern, SearchOption.TopDirectoryOnly);
            var destinationPath = _Destination.FullName;
            Directory.CreateDirectory(destinationPath);

            ProcessFiles(files, async, sourceFile => SyncFile(destinationPath, sourceFile, throwOnError));
        }

        /// <summary>
        /// Removes files from the destination that do not exist in the source
        /// </summary>
        /// <param name="pattern">The pattern.</param>
        /// <param name="throwOnError">if set to <c>true</c> exceptions will be thrown and not swallowed.</param>
        /// <param name="async">if set to <c>true</c> the method will perform the copy asynchronously.</param>
        protected void DeleteDestinationFilesNotInSource(string pattern, bool throwOnError = false, bool async = false)
        {
            var sourceFiles = _SourceDirectory.EnumerateFiles(pattern, SearchOption.TopDirectoryOnly);
            var existingSourceFiles = new HashSet<string>();
            foreach (var sourceFile in sourceFiles)
            {
                existingSourceFiles.Add(sourceFile.FullName);
            }

            var filesToDelete = new List<FileInfo>();
            // Delete all files in the destination that are not in the source
            var destinationFiles = _Destination.EnumerateFiles(pattern, SearchOption.TopDirectoryOnly);
            foreach (var destinationFile in destinationFiles)
            {
                var sourceFileToFind = Path.Combine((_SourceDirectory.FullName), destinationFile.Name);
                if (!existingSourceFiles.Contains(sourceFileToFind))
                {
                    Log("Source does not contain: " + sourceFileToFind);
                    filesToDelete.Add(destinationFile);
                }
            }

            ProcessFiles(filesToDelete, async, fileInfo => DeleteFile(fileInfo, throwOnError));
        }

        protected void OnBeforeCopy(BeforeCopyEventArgs e)
        {
            EventHandler<BeforeCopyEventArgs> handler = BeforeCopy;
            if (handler != null)
                handler(this, e);
        }

        private static void ProcessFiles(IEnumerable<FileInfo> files, bool async, Action<FileInfo> action)
        {
            // Setup parallelization
            var cancellationTokenSource = new CancellationTokenSource();
            var parallelOptions = new ParallelOptions
                                      {
                                          CancellationToken = cancellationTokenSource.Token,
                                          MaxDegreeOfParallelism = Environment.ProcessorCount * 4
                                      };

            var task = Task.Factory.StartNew(() =>
                                                 {
                                                     try
                                                     {
                                                         Parallel.ForEach(files, parallelOptions, (sourceFile, loopState) =>
                                                                                                      {
                                                                                                          if (parallelOptions.CancellationToken.IsCancellationRequested)
                                                                                                          {
                                                                                                              loopState.Stop();
                                                                                                          }

                                                                                                          action(sourceFile);
                                                                                                      });
                                                     }
                                                     catch (OperationCanceledException)
                                                     {
                                                     }
                                                 });
            if (!async)
            {
                task.Wait();
            }
        }

        private void SyncFile(string destinationPath, FileInfo sourceFile, bool throwOnError)
        {
            var destinationFile = new FileInfo(Path.Combine(destinationPath, Path.GetFileName(sourceFile.FullName)));
            bool canSync = false;
            FileCopyReason reason = FileCopyReason.None;
            if (!FileSystemUtils.Exists(destinationFile))
            {
                canSync = true;
                reason = FileCopyReason.New;
            }
            else
            {
                if (sourceFile.LastWriteTimeUtc > destinationFile.LastWriteTimeUtc)
                {
                    canSync = true;
                    reason = FileCopyReason.Updated;
                }
            }
            if (canSync)
            {
                Log("Copying " + sourceFile.FullName + " to " + destinationFile.FullName);
                Exception exception = null;
                try
                {
                    var args = new BeforeCopyEventArgs(sourceFile, destinationFile, reason);
                    OnBeforeCopy(args);
                    File.Copy(args.SourceFile.FullName, args.DestinationFile.FullName, true);
                }
                catch (IOException ioException)
                {
                    exception = ioException;
                }
                catch (UnauthorizedAccessException unauthorizedAccessException)
                {
                    exception = unauthorizedAccessException;
                }
                if (exception != null)
                {
                    Log("Error " + exception);
                    if (throwOnError)
                    {
                        throw exception;
                    }
                }
            }
            else
            {
                Log("Skipping " + sourceFile.FullName + " to " + destinationFile.FullName);
            }
        }

        private static void DeleteFile(FileInfo fileInfo, bool throwOnError)
        {
            Exception exception = null;
            try
            {
                Log("Deleting file: " + fileInfo.FullName);
                fileInfo.Delete();
            }
            catch (IOException ioException)
            {
                exception = ioException;
            }
            catch (UnauthorizedAccessException unauthorizedAccessException)
            {
                exception = unauthorizedAccessException;
            }
            if (exception != null)
            {
                Log("Error " + exception);
                if (throwOnError)
                {
                    throw exception;
                }
            }
        }

        [Conditional("TRACE")]
        private static void Log(string message)
        {
            Trace.WriteLine(message);
        }
    }

    internal enum FileCopyReason
    {
        None,
        New,
        Updated
    }

    internal class BeforeCopyEventArgs : EventArgs
    {
        public FileInfo SourceFile { get; private set; }
        public FileInfo DestinationFile { get; set; }
        public FileCopyReason FileCopyReason { get; private set; }

        public BeforeCopyEventArgs( FileInfo sourceFile, FileInfo destinationFile, FileCopyReason fileCopyReason )
        {
            SourceFile = sourceFile;
            DestinationFile = destinationFile;
            FileCopyReason = fileCopyReason;
        }
    }
}
