﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;

using Extensibility;

using EnvDTE;

using EnvDTE80;

using Shimmy.Api;
using Shimmy.Api.Utils;
using Shimmy.Lib;
using Shimmy.Lib.Dte;
using Shimmy.Properties;

namespace Shimmy
{
    /// <summary>
    /// The object for implementing an Add-in.
    /// </summary>
    /// <seealso class="IDTExtensibility2"/>
    public class VsEnvironmentHook : IDTExtensibility2, IDTCommandTarget
    {
        private DTE2 _ApplicationObject;
        private ApplicationEvents _ApplicationEvents;
        private AddIn _AddInInstance;
        private PluginEnvironment _Environment;

        /// <summary>
        /// Implements the constructor for the Add-in object. Place your initialization code within this method.
        /// </summary>
        public VsEnvironmentHook()
        {
        }

        /// <summary>
        /// Implements the OnConnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being loaded.
        /// </summary>
        /// <param name="application">Root object of the host application.</param>
        /// <param name="connectMode">Describes how the Add-in is being loaded.</param>
        /// <param name="addInInst">Object representing this Add-in.</param>
        /// <param name="custom">Array of parameters that are host application specific.</param>
        /// <seealso class="IDTExtensibility2"/>
        public void OnConnection(object application, ext_ConnectMode connectMode, object addInInst, ref Array custom)
        {
            _ApplicationObject = (DTE2)application;
            _AddInInstance = (AddIn)addInInst;
            
            // Create references to event objects
            _ApplicationEvents = new ApplicationEvents
            {
                BuildEvents = _ApplicationObject.Events.BuildEvents,
                CommandEvents = _ApplicationObject.Events.CommandEvents,
                DebuggerEvents = _ApplicationObject.Events.DebuggerEvents,
                DocumentEvents = _ApplicationObject.Events.DocumentEvents,
                DteEvents = _ApplicationObject.Events.DTEEvents,
                FindEvents = _ApplicationObject.Events.FindEvents,
                OutputWindowEvents = _ApplicationObject.Events.OutputWindowEvents,
                ProjectItemsEvents = new ProjectItemsEventsWrapper(_ApplicationObject),
                SelectionEvents = _ApplicationObject.Events.SelectionEvents,
                SolutionEvents = _ApplicationObject.Events.SolutionEvents,
                SolutionItemsEvents = _ApplicationObject.Events.SolutionItemsEvents,
                TaskListEvents = _ApplicationObject.Events.TaskListEvents,
                TextEditorEvents = _ApplicationObject.Events.TextEditorEvents,
                WindowEvents = _ApplicationObject.Events.WindowEvents
            };

            // Wire up the default error handler
            Api.Utils.ErrorHandler.Handler = DefaultErrorHandler.HandleError;

            // If we are starting up, we'll initialize in the StartupComplete method
            if (connectMode != ext_ConnectMode.ext_cm_Startup)
            {
                // Load all extensions
                Initialize();
            }
        }

        /// <summary>
        /// Implements the OnDisconnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being unloaded.
        /// </summary>
        /// <param name="disconnectMode">Describes how the Add-in is being unloaded.</param>
        /// <param name="custom">Array of parameters that are host application specific.</param>
        /// <seealso class="IDTExtensibility2"/>
        public void OnDisconnection(ext_DisconnectMode disconnectMode, ref Array custom)
        {
            Unload();
        }

        /// <summary>
        /// Implements the OnAddInsUpdate method of the IDTExtensibility2 interface. Receives notification when the collection of Add-ins has changed.
        /// </summary>
        /// <param name="custom">Array of parameters that are host application specific.</param>
        /// <seealso class="IDTExtensibility2"/>
        public void OnAddInsUpdate(ref Array custom)
        {
        }

        /// <summary>
        /// Implements the OnStartupComplete method of the IDTExtensibility2 interface. Receives notification that the host application has completed loading.
        /// </summary>
        /// <param name="custom">Array of parameters that are host application specific.</param>
        /// <seealso class="IDTExtensibility2"/>
        public void OnStartupComplete(ref Array custom)
        {
            Initialize();
        }

        /// <summary>
        /// Implements the OnBeginShutdown method of the IDTExtensibility2 interface. Receives notification that the host application is being unloaded.
        /// </summary>
        /// <param name="custom">Array of parameters that are host application specific.</param>
        /// <seealso class="IDTExtensibility2"/>
        public void OnBeginShutdown(ref Array custom)
        {
        }

        /// <summary>Implements the QueryStatus method of the IDTCommandTarget interface. This is called when the command's availability is updated</summary>
        /// <param name="commandName">The name of the command to determine state for.</param>
        /// <param name="neededText">Text that is needed for the command.</param>
        /// <param name="status">The state of the command in the user interface.</param>
        /// <param name="commandText">Text requested by the neededText parameter.</param>
        /// <seealso cref="Exec" />
        public void QueryStatus(string commandName, vsCommandStatusTextWanted neededText, ref vsCommandStatus status, ref object commandText)
        {
            if (_Environment.CommandHandlerRegistry.ContainsKey(commandName))
            {
                var handlers = _Environment.CommandHandlerRegistry[commandName];
                if (handlers.Count > 0)
                {
                    try
                    {
                        foreach (var commandHandler in handlers)
                        {
                            commandHandler.QueryStatus(_Environment.Context, commandName.Replace(_AddInInstance.ProgID + ".", string.Empty), neededText, ref status, ref commandText);
                        }
                    }
                    catch (Exception exception)
                    {
                        DefaultErrorHandler.HandleError(exception, string.Format(Resources.ErrorQueryingCommand, commandName, exception));
                    }
                }
            }
        }

        /// <summary>Implements the Exec method of the IDTCommandTarget interface. This is called when the command is invoked.</summary>
        /// <param name='commandName'>The name of the command to execute.</param>
        /// <param name='executeOption'>Describes how the command should be run.</param>
        /// <param name='varIn'>Parameters passed from the caller to the command handler.</param>
        /// <param name='varOut'>Parameters passed from the command handler to the caller.</param>
        /// <param name='handled'>Informs the caller if the command was handled or not.</param>
        /// <seealso cref='QueryStatus' />
        public void Exec(string commandName, vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool handled)
        {
            if (_Environment.CommandHandlerRegistry.ContainsKey(commandName))
            {
                var handlers = _Environment.CommandHandlerRegistry[commandName];
                if (handlers.Count > 0)
                {
                    try
                    {
                        foreach (var commandHandler in handlers)
                        {
                            commandHandler.Exec(_Environment.Context, commandName.Replace(_AddInInstance.ProgID + ".", string.Empty), executeOption, ref varIn, ref varOut, ref handled);
                        }
                    }
                    catch (Exception exception)
                    {
                        DefaultErrorHandler.HandleError(exception, string.Format(Resources.ErrorExecutingCommand, commandName, exception));
                    }
                }

            }
        }

        private void Initialize()
        {
            try
            {
                // Register the context
                var context = new PluginContext
                                  {
                                      AddIn = _AddInInstance,
                                      Application = _ApplicationObject,
                                      ApplicationEvents = _ApplicationEvents
                                  };
                _Environment = new PluginEnvironment(context, GlobalSettings.Load());
                _Environment.PluginImportSyncComplete += PluginImportSyncComplete;
                _Environment.Initialize();
            }
            catch (Exception e)
            {
                ErrorHandler.HandleError(e.Message);
            }
        }

        private void Unload()
        {
            _Environment.Unload();
            _Environment.PluginImportSyncComplete -= PluginImportSyncComplete;
        }

        private static void PluginImportSyncComplete(object sender, PluginImportEventArgs e)
        {
            // Request permission to use any new or updated plugins
            ProcessNewAndUpdatedPlugins(e.Manager);
        }

        private static void ProcessNewAndUpdatedPlugins(PluginImportManager importManager)
        {
            var updatedPlugins = importManager.GetPendingUpdates();
            var newPlugins = importManager.GetPendingPlugins();
            if (updatedPlugins != null && updatedPlugins.Count > 0)
            {
                foreach (var updatedPlugin in updatedPlugins)
                {
                    string realPluginName = updatedPlugin.Value;
                    if (MessageBox.Show(string.Format(Resources.UpdatedPluginFoundMessage, realPluginName), Resources.UpdatedPluginFoundCaption, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                    {
                        importManager.AllowPlugin(updatedPlugin.Key);
                    }
                }
            }

            if (newPlugins != null && newPlugins.Count > 0)
            {
                foreach (var newPlugin in newPlugins)
                {
                    string realPluginName = newPlugin.Value;
                    if (MessageBox.Show(string.Format(Resources.NewPluginFoundMessage, realPluginName), Resources.NewPluginFoundCaption, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                    {
                        importManager.AllowPlugin(newPlugin.Key);
                    }
                }
            }
        }
    }
}